# -*- coding: utf-8 -*-
# @Time : 4/1/2024 15:36 
# @Author : luoshuai
# @File : define.py 
# @Software: PyCharm
import os
import socket
import sys
import time

ENABLE_DEBUG = sys.platform.startswith('win')


class UpdateCode(object):
    SUCCESS = 0
    FAILED = -1
    EXCEPTION = -2
    RESTART_AGENT_FAILED = 233
    RESTART_AGENT_ERROR = 234


class Log(object):
    logger = None

    @staticmethod
    def get_log_file():
        if ENABLE_DEBUG:
            return 'update.log'
        else:
            return '/tmp/update.log'

    @staticmethod
    def info(*args, **kwargs):
        if Log.logger:
            Log.logger.info(*args, **kwargs)

    @staticmethod
    def warn(*args, **kwargs):
        if Log.logger:
            Log.logger.warn(*args, **kwargs)

    @staticmethod
    def debug(*args, **kwargs):
        if Log.logger:
            Log.logger.debug(*args, **kwargs)

    @staticmethod
    def error(*args, **kwargs):
        if Log.logger:
            Log.logger.error(*args, **kwargs)


class OTAError(Exception):
    def __init__(self, info, code=UpdateCode.FAILED):
        Exception.__init__(self)
        self.info = info
        self.code = code

    def __str__(self):
        return '(%d)%s' % (self.code, self.info)

    def __repr__(self):
        return self.__str__()


class OTAResult(object):
    def __init__(self, info, code=UpdateCode.FAILED):
        self.info = info
        self.code = code

    def __str__(self):
        return '(%d)%s' % (self.code, self.info)

    def __repr__(self):
        return self.__str__()


class AgentBasic(object):
    WG_WORKER_AGENT_PORT = 11001
    WG_UPDATER_AGENT_PORT = 20001

    WG_CMD_RESTART_AGNET = "rq:restart-agentbasic"
    WG_CMD_RESTART_AGNET_OK = "rq:restart-agentbasic=1"

    AGENT_DIR = '/opt/agent'
    AGENT_MODULECONFIG_XML = '/opt/agent/ModuleConfig.xml'

    @staticmethod
    def get_module_config():
        if ENABLE_DEBUG:
            return 'ModuleConfig.xml'
        else:
            return AgentBasic.AGENT_MODULECONFIG_XML

    @staticmethod
    def restart():
        rs = UpdateCode.RESTART_AGENT_FAILED
        try_num = 2
        client = None
        try:
            client = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            client.settimeout(10)
            client.connect(('127.0.0.1', AgentBasic.WG_WORKER_AGENT_PORT))
            for i in range(try_num):
                client.sendall(AgentBasic.WG_CMD_RESTART_AGNET)
                response = client.recv(1024)
                if response != AgentBasic.WG_CMD_RESTART_AGNET_OK:
                    time.sleep(1)
                    continue
                else:
                    rs = UpdateCode.SUCCESS
                    break
        except Exception as e:
            Log.warn("Failed to restart agent!" + repr(e))
            rs = UpdateCode.RESTART_AGENT_ERROR
        finally:
            if client:
                client.close()
        return rs

    @staticmethod
    def entry():
        try:
            if ENABLE_DEBUG:
                return
            cmd = "cd %s && nohup python watchdog.py >/dev/null 2>&1 &" % AgentBasic.AGENT_DIR
            Log.info('entry agentbasic: ' + cmd)
            os.system(cmd)
            time.sleep(1)
        except Exception as e:
            Log.warn("Failed to entry agent!" + repr(e))

    @staticmethod
    def exit():
        try:
            if ENABLE_DEBUG:
                return
            cmd = "ps -ef | grep -wE \"agentbasic|watchdog.py\" | grep -v grep | awk '{print $1}' | xargs -r kill -9"
            Log.info('exit agentbasic: ' + cmd)
            os.system(cmd)
            time.sleep(1)
        except Exception as e:
            Log.warn("Failed to exit agent!" + repr(e))


class DeviceOTABase(object):
    def __init__(self, root_path, tag1, tag2, tag3):
        self.root_path = root_path
        self.tag1 = tag1
        self.tag2 = tag2
        self.tag3 = tag3

        self.result_code = UpdateCode.FAILED
        self.result_message = ''

    def run(self):
        raise NotImplementedError('DeviceOTABase.run is not implemented !')
